﻿using System.Data;
using System.Linq;
using System.ServiceModel.DomainServices.EntityFramework;
using System.ServiceModel.DomainServices.Hosting;
using System.ServiceModel.DomainServices.Server;
using ScipBe.Demo.RiaCodeGen.Web.Domain;

namespace ScipBe.Demo.RiaCodeGen.Web.Services
{
    [EnableClientAccess()]
    public class ProductService : LinqToEntitiesDomainService<NorthwindEntities>
    {
        [Query(IsDefault = true)]
        public IQueryable<Product> GetProducts()
        {
            var products = ObjectContext.Products;
            return products;
        }

        public IQueryable<Product> GetProductsByName(string searchText)
        {
            var products = ObjectContext.Products.Where(p => p.ProductName.Contains(searchText));
            return products;
        }

        public void InsertProduct(Product product)
        {
            if ((product.EntityState != EntityState.Detached))
            {
                ObjectContext.ObjectStateManager.ChangeObjectState(product, EntityState.Added);
            }
            else
            {
                ObjectContext.Products.AddObject(product);
            }
        }

        public void UpdateProduct(Product currentProduct)
        {
            ObjectContext.Products.AttachAsModified(currentProduct, ChangeSet.GetOriginal(currentProduct));
        }

        public void DeleteProduct(Product product)
        {
            if ((product.EntityState != EntityState.Detached))
            {
                ObjectContext.ObjectStateManager.ChangeObjectState(product, EntityState.Deleted);
            }
            else
            {
                ObjectContext.Products.Attach(product);
                ObjectContext.Products.DeleteObject(product);
            }
        }
    }
}


