﻿// --------------------------------------------------------------------------------------------
// Stefan Cruysberghs
// http://www.scip.be
// Article: Customize WCF RIA Services code generation
// http://www.scip.be/index.php?Page=ArticlesNET36
// --------------------------------------------------------------------------------------------
using System;
using System.Diagnostics;
using System.ServiceModel;
using System.ServiceModel.DomainServices.Client;

namespace ScipBe.Demo.RiaCodeGen.Framework
{
    public abstract class MyDomainContext : DomainContext
    {
        public MyDomainContext(DomainClient domainClient) : base(domainClient)
        {
            var channelFactory = (ChannelFactory)DomainClient.GetType().GetProperty("ChannelFactory").GetValue(DomainClient, null);

            var timeout = new TimeSpan(0, 5, 00);
            if (Debugger.IsAttached)
            {
                timeout = new TimeSpan(0, 15, 00);
            }

            var endpointBinding = channelFactory.Endpoint.Binding;
            endpointBinding.OpenTimeout = timeout;
            endpointBinding.CloseTimeout = timeout;
            endpointBinding.SendTimeout = timeout;
        }

        public LoadOperation Load(EntityQuery query, Action<LoadOperation> callback)
        {
            return Load(query, LoadBehavior.RefreshCurrent, callback, null);
        }

        public override LoadOperation Load(EntityQuery query, LoadBehavior loadBehavior, Action<LoadOperation> callback, object userState)
        {
            Action<LoadOperation> newCallback =
                loadOperation =>
                    {
                        MyWebContext.DecreaseBusyCounter();
                        if (callback != null)
                        {
                            callback.Invoke(loadOperation);
                        }
                    };

            MyWebContext.DecreaseBusyCounter();
            return base.Load(query, loadBehavior, newCallback, userState);
        }

        public override SubmitOperation SubmitChanges(Action<SubmitOperation> callback, object userState)
        {
            Action<SubmitOperation> newCallback =
                submitOperation =>
                {
                    MyWebContext.DecreaseBusyCounter();
                    if (callback != null)
                    {
                        callback.Invoke(submitOperation);
                    }
                };

            MyWebContext.DecreaseBusyCounter();
            return base.SubmitChanges(newCallback, userState);
        }

        public override InvokeOperation<TValue> InvokeOperation<TValue>(string operationName, Type returnType, System.Collections.Generic.IDictionary<string, object> parameters, bool hasSideEffects, Action<InvokeOperation<TValue>> callback, object userState)
        {
            Action<InvokeOperation<TValue>> newCallback =
                invokeOperation =>
                {
                    MyWebContext.DecreaseBusyCounter();
                    if (callback != null)
                    {
                        callback.Invoke(invokeOperation);
                    }
                };

            MyWebContext.DecreaseBusyCounter();
            return base.InvokeOperation<TValue>(operationName, returnType, parameters, hasSideEffects, newCallback, userState);
        }

        public override InvokeOperation InvokeOperation(string operationName, Type returnType, System.Collections.Generic.IDictionary<string, object> parameters, bool hasSideEffects, Action<InvokeOperation> callback, object userState)
        {
            Action<InvokeOperation> newCallback =
                invokeOperation =>
                {
                    MyWebContext.DecreaseBusyCounter();
                    if (callback != null)
                    {
                        callback.Invoke(invokeOperation);
                    }
                };

            MyWebContext.DecreaseBusyCounter();
            return base.InvokeOperation(operationName, returnType, parameters, hasSideEffects, newCallback, userState);
        }
    }
}